@@ -313,7 +313,7 @@ int32_t usbi_control_xfer(struct usbi_dev_handle *devh,int requesttype,
 	return ret;
 }
 
-int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbuf,
+static int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbuf,
                 int32_t *cfglen)
 {
 	char buf[8];
@@ -322,9 +322,11 @@ int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbu
 	struct usb_config_desc cfg_desc;
 	uint32_t count;
 
+	/* The timeout has been bumped from 100ms to 1000ms to work better with */
+	/* virtual machines and finicky devices... */
 	ret = usbi_control_xfer(devh, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
 			(USB_DESC_TYPE_CONFIG << 8) + cfg, 0, buf,
-			8, 100);
+			8, 1000);
 	if (ret < 0) {
 		usbi_debug(NULL, 1, "usbi_control_xfer fail");
 		return ret;
@@ -339,9 +341,11 @@ int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbu
 		return OPENUSB_NO_RESOURCES;
 	}
 
+	/* The timeout has been bumped from 100ms to 1000ms to work better with */
+	/* virtual machines and finicky devices... */
 	ret = usbi_control_xfer(devh, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR,
 			(USB_DESC_TYPE_CONFIG << 8) + cfg, 0, newbuf,
-			cfg_desc.wTotalLength, 100);
+			cfg_desc.wTotalLength, 1000);
 
 	if (ret < 0) {
 		free(newbuf);
@@ -357,7 +361,7 @@ int32_t usbi_get_config_desc(struct usbi_dev_handle *devh, int cfg, char **cfgbu
 	return 0;
 }
 
-void usbi_free_cfg(char *buf)
+static void usbi_free_cfg(char *buf)
 {
 	if (buf == NULL) {
 		return;
@@ -372,7 +376,7 @@ void usbi_free_cfg(char *buf)
  * But, we can't judge if a READ request is sent to an out endpoint, and
  * vice vesa.
  */
-int32_t check_req_valid(openusb_request_handle_t req, 
+static int32_t check_req_valid(openusb_request_handle_t req, 
 	struct usbi_dev_handle *dev)
 {
 	openusb_transfer_type_t type = req->type;
@@ -1023,7 +1027,7 @@ int32_t openusb_poll(uint32_t num_reqs,openusb_request_handle_t * handles,
 	pthread_mutex_lock(&ph->complete_lock);
 	list_for_each_entry(io,&ph->complete_list,list) {
 	/* safe */
-		if(io) {
+		if (io) {
 			/*get the first io on the complete list */
 			usbi_debug(ph, 4, "complete list: %p\n",
 				io->req);
@@ -1092,7 +1096,7 @@ struct usbi_multi_req_args {
  * process individual previously submitted openusb_request_handle
  *
  */
-int32_t multi_req_callback(openusb_request_handle_t req)
+static int32_t multi_req_callback(openusb_request_handle_t req)
 {
 	openusb_multi_request_handle_t mreq;
 	struct usbi_multi_request *mi_req;
@@ -1170,7 +1174,7 @@ int32_t multi_req_callback(openusb_request_handle_t req)
  *		      ...
  *                    --> usbi_multi_req_args
  */
-int process_multi_request(void *arg)
+static int process_multi_request(void *arg)
 {
 	struct usbi_multi_request *mi_req = (struct usbi_multi_request *)arg;
 	int i;
@@ -1368,11 +1372,13 @@ loop:
 		struct usbi_multi_req_args *pargs, *tmp;
 
 		list_for_each_entry_safe(pargs, tmp, &mi_req->req_head, list) {
-			pthread_mutex_unlock(&mi_req->lock);
-			openusb_abort(pargs->req);
-			pthread_mutex_lock(&mi_req->lock);
-			free(pargs->req); /*FIXME: should be here ??? */
-			free(pargs);
+			if (pargs) {
+				pthread_mutex_unlock(&mi_req->lock);
+				openusb_abort(pargs->req);
+				pthread_mutex_lock(&mi_req->lock);
+				free(pargs->req); /*FIXME: should be here ??? */
+				free(pargs);
+			}
 		}
 
 		pthread_mutex_unlock(&mi_req->lock);
@@ -1459,7 +1465,7 @@ int32_t openusb_start(openusb_multi_request_handle_t handle)
 	return ret;
 }
 
-int32_t usbi_add_or_stop(openusb_multi_request_handle_t handle, int flag)
+static int32_t usbi_add_or_stop(openusb_multi_request_handle_t handle, int flag)
 {
 	struct usbi_multi_request *mreq;
 	struct usbi_dev_handle *hdev;
@@ -1477,7 +1483,7 @@ int32_t usbi_add_or_stop(openusb_multi_request_handle_t handle, int flag)
 	mreq = NULL;
 	list_for_each_entry(mreq, &hdev->m_head, list) {
 	/* safe */
-		if(mreq->mreq == handle) {
+		if (mreq && (mreq->mreq == handle)) {
 			break;
 		}
 	}
@@ -1504,7 +1510,7 @@ int32_t usbi_add_or_stop(openusb_multi_request_handle_t handle, int flag)
  *    the previous buffers are safe to use. All the internal processing will
  *    use these buffers until user's callback is called.
  */
-int32_t openusb_add(openusb_multi_request_handle_t handle)
+static int32_t openusb_add(openusb_multi_request_handle_t handle)
 {
 	return (usbi_add_or_stop(handle, USBI_MREQ_NEW_BUF));
 }
