@@ -38,7 +38,7 @@ int testhotplug = 0;
 #define ISOC_PKT_NUM 12
 #define ISOC_PKT_LEN 128
 
-void print_endpoint(struct usb_endpoint_desc *ep)
+static void print_endpoint(struct usb_endpoint_desc *ep)
 { 
 	printf("      bEndpointAddress: %02xh\n", ep->bEndpointAddress);
 	printf("      bmAttributes:     %02xh\n", ep->bmAttributes);
@@ -49,8 +49,8 @@ void print_endpoint(struct usb_endpoint_desc *ep)
 	printf("\n");
 }
 
-void print_interface(openusb_devid_t devid, int cfgidx, int ifcidx,int alt,
-		struct usb_interface_desc *intf)
+static void print_interface(openusb_devid_t devid, int cfgidx, int
+		ifcidx,int alt, struct usb_interface_desc *intf)
 { 
 	int i;
 	int ret;
@@ -80,7 +80,7 @@ void print_interface(openusb_devid_t devid, int cfgidx, int ifcidx,int alt,
 	}
 }
 
-void print_configuration(openusb_devid_t devid, int cfgidx,
+static void print_configuration(openusb_devid_t devid, int cfgidx,
 		struct usb_config_desc *cfg)
 {
 	int i;
@@ -114,7 +114,7 @@ void print_configuration(openusb_devid_t devid, int cfgidx,
 }
 
 /* print device descriptors */
-void print_device(openusb_devid_t devid, int indent)
+static void print_device(openusb_devid_t devid, int indent)
 {
 	struct usb_device_desc dev;
 	unsigned char *buf=NULL;
@@ -161,10 +161,10 @@ void print_device(openusb_devid_t devid, int indent)
 }
 
 
-void event_cb(openusb_handle_t handle, openusb_devid_t devid,
+static void event_cb(openusb_handle_t handle, openusb_devid_t devid,
 		openusb_event_t event, void *arg)
 {
-	char *evstring;
+	const char *evstring;
 	switch(event) {
 	case USB_ATTACH:
 		evstring = "USB_ATTACH";
@@ -178,6 +178,15 @@ void event_cb(openusb_handle_t handle, openusb_devid_t devid,
 	case USB_RESUME:
 		evstring = "USB_RESUME";
 		break;
+	case USB_HC_ATTACH:
+		evstring = "USB_HC_ATTACH";
+		break;
+	case USB_HC_REMOVE:
+		evstring = "USB_HC_REMOVE";
+		break;
+	case USB_COLDPLUG_COMPLETED:
+		evstring = "USB_COLDPLUG_COMPLETED";
+		break;
 	default:
 		evstring = "Unknown";
 		break;
@@ -186,7 +195,7 @@ void event_cb(openusb_handle_t handle, openusb_devid_t devid,
 		(long long)handle, (long long)devid, evstring, arg);
 }
 
-int convert_string(char *buf, usb_string_desc_t *st, int buflen)
+static int convert_string(char *buf, usb_string_desc_t *st, int buflen)
 {
 	int di, si;
 	unsigned char *tbuf = (unsigned char *)st;
@@ -208,7 +217,7 @@ int convert_string(char *buf, usb_string_desc_t *st, int buflen)
 	return di;
 }
 
-void dump_dev_data(openusb_dev_data_t *pdev)
+static void dump_dev_data(openusb_dev_data_t *pdev)
 {
 	struct usb_device_desc *pdesc;
 	int i;
@@ -229,7 +238,7 @@ void dump_dev_data(openusb_dev_data_t *pdev)
 	pdesc = &pdev->dev_desc;	
 
 	printf("Device descriptor:\n");
-	printf("\tclass:%02x subclass:%02x vid:%04hx pid:%04hx\n",
+	printf("\tclass: %02x subclass: %02x vid: %04hx pid: %04hx\n",
 			pdesc->bDeviceClass,
 			pdesc->bDeviceSubClass,pdesc->idVendor,
 			pdesc->idProduct);
@@ -275,7 +284,7 @@ void dump_dev_data(openusb_dev_data_t *pdev)
 #define CTRL_LEN 0xab
 
 /* test SYNC control xfer */
-int test_ctrl_sync(openusb_dev_handle_t devh)
+static int test_ctrl_sync(openusb_dev_handle_t devh)
 {
 	openusb_ctrl_request_t ctrl;
 	int ret;
@@ -299,7 +308,7 @@ int test_ctrl_sync(openusb_dev_handle_t devh)
 	ret = openusb_ctrl_xfer(devh, 0, 0, &ctrl);
 	if (ret != 0) {
 		openusb_free_devid_list(devids);
-		printf("ctrl xfer fail:%s\n", openusb_strerror(ret));
+		printf("ctrl xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -328,7 +337,7 @@ int test_ctrl_sync(openusb_dev_handle_t devh)
  * Use EZ-USB FX2 as the test device
  * load bulkloop firmware
  */
-int test_bulk_sync(openusb_handle_t devh)
+static int test_bulk_sync(openusb_handle_t devh)
 {
 	unsigned char bulkdata[BULK_DATA_LEN];
 	unsigned char bulkrd[BULK_DATA_LEN];
@@ -350,7 +359,7 @@ int test_bulk_sync(openusb_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -358,7 +367,7 @@ int test_bulk_sync(openusb_handle_t devh)
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -368,7 +377,7 @@ int test_bulk_sync(openusb_handle_t devh)
 	ret = openusb_bulk_xfer(devh, 0, 2, &bulk);
 
 	if (ret != 0) {
-		printf("BULK sync xfer test fail:%s\n", openusb_strerror(ret));
+		printf("BULK sync xfer test fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 	printf("bulk sync xfer result.status = %d,xfer_bytes=%d, ret=%d\n",
@@ -378,7 +387,7 @@ int test_bulk_sync(openusb_handle_t devh)
 	bulk.payload = bulkrd;
 	ret = openusb_bulk_xfer(devh, 0, 0x86, &bulk);
 	if (ret != 0) {
-		printf("bulk sync xfer fail:%s\n", openusb_strerror(ret));
+		printf("bulk sync xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -405,7 +414,7 @@ int test_bulk_sync(openusb_handle_t devh)
  * Use EZ-USB FX2 as the test device.
  * Load intrsrc or intrloop firmware
  */
-int test_intr_sync(openusb_handle_t devh, int flag)
+static int test_intr_sync(openusb_handle_t devh, int flag)
 {
 	unsigned char bulkdata[BULK_DATA_LEN];
 	unsigned char bulkrd[BULK_DATA_LEN];
@@ -426,7 +435,7 @@ int test_intr_sync(openusb_handle_t devh, int flag)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -434,7 +443,7 @@ int test_intr_sync(openusb_handle_t devh, int flag)
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -445,7 +454,7 @@ int test_intr_sync(openusb_handle_t devh, int flag)
 		ret = openusb_intr_xfer(devh, 0, 2, &intr);
 
 		if (ret != 0) {
-			printf("xfer fail:%s\n", openusb_strerror(ret));
+			printf("xfer fail: %s\n", openusb_strerror(ret));
 			return -1;
 		}
 
@@ -458,7 +467,7 @@ int test_intr_sync(openusb_handle_t devh, int flag)
 	ret = openusb_intr_xfer(devh, 0, 0x86, &intr);
 	
 	if (ret != 0) {
-		printf("intr xfer sync fail:%s\n", openusb_strerror(ret));
+		printf("intr xfer sync fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -484,7 +493,7 @@ int test_intr_sync(openusb_handle_t devh, int flag)
  * Test ISOC sync xfer
  * Use EZ-USB FX2 as test device. Load isoc firmware
  */
-int test_isoc_sync(openusb_handle_t devh)
+static int test_isoc_sync(openusb_handle_t devh)
 {
 	unsigned char bulkdata[ISOC_PKT_NUM*ISOC_PKT_LEN];
 	unsigned char bulkrd[ISOC_PKT_NUM*ISOC_PKT_LEN];
@@ -502,7 +511,7 @@ int test_isoc_sync(openusb_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -510,7 +519,7 @@ int test_isoc_sync(openusb_handle_t devh)
 
 	ret = openusb_set_altsetting(devh,0,3);/* alt 3, depends on the fw */
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -532,7 +541,7 @@ int test_isoc_sync(openusb_handle_t devh)
 	ret = openusb_isoc_xfer(devh, 0, 0x82, &isoc); /* in */
 
 	if (ret != 0) {
-		printf("ISOC xfer fail:%s\n", openusb_strerror(ret));
+		printf("ISOC xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -562,7 +571,7 @@ int test_isoc_sync(openusb_handle_t devh)
 	return 0;
 }
 
-int async_xfer_ctrl_test(openusb_dev_handle_t devh)
+static int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 {
 	unsigned char bulkdata[BULK_DATA_LEN];
 	unsigned char bulkrd[BULK_DATA_LEN];
@@ -599,7 +608,7 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -607,7 +616,7 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -617,7 +626,7 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 
 	ret = openusb_xfer_aio(req);
 	if (ret != 0) {
-		printf("xfer fail:%s\n", openusb_strerror(ret));
+		printf("xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -642,7 +651,7 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 			ret);
 
 	if (ret != 0) {
-		printf("Ctrl async xfer fail:%s\n", openusb_strerror(ret));
+		printf("Ctrl async xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -650,7 +659,7 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
 	while(1) {
 		ret = openusb_poll(1, &req, &completed);
 		if (ret != 0) {
-			printf("async xfer poll:%s\n", openusb_strerror(ret));
+			printf("async xfer poll: %s\n", openusb_strerror(ret));
 			return -1;
 		}
 
@@ -683,7 +692,8 @@ int async_xfer_ctrl_test(openusb_dev_handle_t devh)
  * Async xfer test for INTR, BULK, ISOC
  * Load different firmware accordingly
  */
-int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int flag)
+static int async_xfer_test(openusb_handle_t devh,
+		openusb_transfer_type_t type, int flag)
 {
 	unsigned char bulkdata[BULK_DATA_LEN];
 	unsigned char bulkrd[BULK_DATA_LEN];
@@ -743,7 +753,7 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -756,7 +766,7 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 	}
 
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -819,7 +829,7 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 		}
 
 		if (ret != 0) {
-			printf("xfer fail:%s\n", openusb_strerror(ret));
+			printf("xfer fail: %s\n", openusb_strerror(ret));
 			return -1;
 		}
 
@@ -901,7 +911,7 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 
 
 	if (ret != 0) {
-		printf("async xfer fail:%s\n", openusb_strerror(ret));
+		printf("async xfer fail: %s\n", openusb_strerror(ret));
 		return -1;
 	}
 
@@ -923,7 +933,7 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 	while(count < loopcnt) {
 		ret = openusb_poll(loopcnt, reqs, &completed);
 		if (ret != 0) {
-			printf("async xfer poll:%s\n", openusb_strerror(ret));
+			printf("async xfer poll: %s\n", openusb_strerror(ret));
 			return -1;
 		}
 		if (completed == NULL) {
@@ -1002,7 +1012,8 @@ int async_xfer_test(openusb_handle_t devh, openusb_transfer_type_t type, int fla
 uint32_t m_len[ISOC_PKT_NUM];
 uint8_t *m_buffers[ISOC_PKT_NUM];
 
-int32_t multi_callback(struct openusb_multi_request_handle *mreq,
+#if 0
+static int32_t multi_callback(struct openusb_multi_request_handle *mreq,
 	uint32_t bufidx, openusb_request_result_t *result)
 {
 	int i;
@@ -1028,7 +1039,7 @@ int32_t multi_callback(struct openusb_multi_request_handle *mreq,
 /*
  * Not mature
  */
-int multi_xfer_test(openusb_dev_handle_t devh)
+static int multi_xfer_test(openusb_dev_handle_t devh)
 {
 	int ret = 0;
 	int i;
@@ -1037,7 +1048,7 @@ int multi_xfer_test(openusb_dev_handle_t devh)
 
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -1045,7 +1056,7 @@ int multi_xfer_test(openusb_dev_handle_t devh)
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		return -1;
@@ -1097,18 +1108,19 @@ int multi_xfer_test(openusb_dev_handle_t devh)
 
 	return ret;
 }
+#endif
 
-int test_get_device_data()
+static int test_get_device_data(void)
 {
 	int ret;
 	openusb_dev_data_t *devdata;
-	int i,j;
+	unsigned int i, j;
 
 	for(j=0;j<busnum;j++) {
 		ret = openusb_get_devids_by_bus(libhandle, bus[j], &devids,
 				&devnum);
 		if(ret < 0) {
-			printf("Error get devids by bus:%s\n",
+			printf("Error get devids by bus: %s\n",
 					openusb_strerror(ret));
 			return -1;
 		}
@@ -1118,15 +1130,15 @@ int test_get_device_data()
 			ret = openusb_get_device_data(libhandle, devids[i], 0,
 					&devdata);
 			if (ret < 0) {
-				printf("Get device(%d) data error:%s\n", i,
+				printf("Get device(%d) data error: %s\n", i,
 						openusb_strerror(ret));
-				return -1;
 			}
-			
-			dump_dev_data(devdata);
-
-			openusb_free_device_data(devdata);
+			else
+			{
+				dump_dev_data(devdata);
 
+				openusb_free_device_data(devdata);
+			}
 		}
 		openusb_free_devid_list(devids);
 	}
@@ -1134,7 +1146,7 @@ int test_get_device_data()
 	return 0;
 }
 
-int test_sync_xfer(openusb_dev_handle_t devh)
+static int test_sync_xfer(openusb_dev_handle_t devh)
 {
 	int ret;
 
@@ -1160,7 +1172,7 @@ int test_sync_xfer(openusb_dev_handle_t devh)
 	return 0;
 }
 
-int test_async_xfer(openusb_dev_handle_t devh)
+static int test_async_xfer(openusb_dev_handle_t devh)
 {
 	int ret;
 
@@ -1179,7 +1191,7 @@ int test_async_xfer(openusb_dev_handle_t devh)
 	return 0;
 }
 
-int advance_xfer_test(void)
+static int advance_xfer_test(void)
 {
 	int ret = 0;
 	openusb_dev_handle_t devh;
@@ -1197,13 +1209,13 @@ int advance_xfer_test(void)
 	ret = openusb_get_devids_by_class(libhandle, 0xff,
 		-1, -1, &devids, &devnum);
 	if(ret < 0) {
-		printf("Error get devids by class:%s\n",openusb_strerror(ret));
+		printf("Error get devids by class: %s\n",openusb_strerror(ret));
 		return -1;
 	}
 
 	ret = openusb_open_device(libhandle,devids[0],0,&devh);
 	if (ret != 0) {
-		printf("Open device(%d) error:%s\n",(int)devids[0],
+		printf("Open device(%d) error: %s\n",(int)devids[0],
 				openusb_strerror(ret));
 		goto err;
 	}
@@ -1212,14 +1224,14 @@ int advance_xfer_test(void)
 
 	ret = openusb_get_devid(devh, &devid);
 	if(ret < 0) {
-		printf("Error get devids by handle:%s\n",openusb_strerror(ret));
+		printf("Error get devids by handle: %s\n",openusb_strerror(ret));
 		goto err;
 	}
 	printf("devh=%x devid=%x\n",(int)devh,(int)devid);
 
 	ret = openusb_get_lib_handle(devh,&libh);
 	if (ret != 0) {
-		printf("Get device(%llu) lib handle error:%s\n",
+		printf("Get device(%llu) lib handle error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1230,7 +1242,7 @@ int advance_xfer_test(void)
 	ret = multi_xfer_test(devh);
 	if(ret !=0) {
 		openusb_free_devid_list(devids);
-		printf("multi_xfer_test error:%s\n", openusb_strerror(ret));
+		printf("multi_xfer_test error: %s\n", openusb_strerror(ret));
 		return -1;
 
 	}
@@ -1255,7 +1267,7 @@ int advance_xfer_test(void)
 #if 1	/* configuration test */
 	ret = openusb_set_configuration(devh,1);
 	if (ret != 0) {
-		printf("Set device(%llu) config error:%s\n",
+		printf("Set device(%llu) config error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1263,7 +1275,7 @@ int advance_xfer_test(void)
 
 	ret = openusb_get_configuration(devh,&cfg);
 	if (ret != 0) {
-		printf("Get device(%llu) config error:%s\n",
+		printf("Get device(%llu) config error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1275,7 +1287,7 @@ int advance_xfer_test(void)
 #if 1	/* claim interface test */
 	ret = openusb_claim_interface(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) claim interface error:%s\n",
+		printf("Device(%llu) claim interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1288,7 +1300,7 @@ int advance_xfer_test(void)
 
 	ret = openusb_set_altsetting(devh,0,0);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) set alt:%s\n",
+		printf("Device(%llu) interface(0) set alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1297,7 +1309,7 @@ int advance_xfer_test(void)
 
 	ret = openusb_get_altsetting(devh,0,&alt);
 	if (ret != 0) {
-		printf("Device(%llu) interface(0) get alt:%s\n",
+		printf("Device(%llu) interface(0) get alt: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1306,7 +1318,7 @@ int advance_xfer_test(void)
 
 	ret = openusb_release_interface(devh,0);
 	if (ret != 0) {
-		printf("Device(%llu) release interface error:%s\n",
+		printf("Device(%llu) release interface error: %s\n",
 				(long long)devids[0],
 				openusb_strerror(ret));
 		goto err;
@@ -1315,7 +1327,7 @@ int advance_xfer_test(void)
 	   ret = openusb_reset(devh);
 	   if (ret != 0) {
 	   openusb_free_devid_list(devids);
-	   printf("Device(%llu) reset error:%s\n",devids[0],
+	   printf("Device(%llu) reset error: %s\n",devids[0],
 	   openusb_strerror(ret));
 	   return -1;
 	   }
@@ -1329,7 +1341,7 @@ err:
 	return ret;
 }
 
-void usage(char *prog)
+static void usage(char *prog)
 {
 	printf("usage:\n");
 	printf("%s\n", prog);
@@ -1349,9 +1361,9 @@ void usage(char *prog)
 }
 
 
-int parse_option(int argc, char *argv[])
+static int parse_option(int argc, char *argv[])
 {
-	char c;
+	int c;
 	while((c = getopt(argc, argv, "t:lmaps")) != -1) {
 		switch (c) {
 			case 't':
@@ -1407,14 +1419,14 @@ err1:
  * Get all buses on the system.
  * Get every device's data and print them. 
  */
-int basic_test(void)
+static int basic_test(void)
 {
 	int ret;
 	uint32_t flags = 0;
 
 	ret = openusb_init(flags, &libhandle);
 	if(ret < 0) {
-		printf("error init\n");
+		printf("error init: %s (%d)\n", openusb_strerror(ret), ret);
 		exit(1);
 	}
 	printf("lib handle=%llu \n", (long long)libhandle);
@@ -1440,7 +1452,7 @@ int basic_test(void)
 		return -1;
 	}
 
-	printf("openusb_set_default_timeout : PASS\n");
+	printf("openusb_set_default_timeout: PASS\n");
 
 	/*get buses */
 	ret = openusb_get_busid_list(libhandle,&bus,&busnum);
@@ -1455,7 +1467,7 @@ int basic_test(void)
 	return 0;
 }
 
-void cleanup(void)
+static void cleanup(void)
 {
 	openusb_free_busid_list(bus);
 
